#include "delay.h"
#include "uart.h"
#include "stdio.h"
extern void SysClockConfig(void);

/*Variable to hold the measured pulse length*/
uint32_t period;

/*variable to hold the measured distance*/
float distance;

int main(void)
{



	/*Enable float point hardware*/
	SCB->CPACR |= ((3UL << 10*2)|(3UL << 11*2));
	/*Set the clock to 100MHz*/
	SysClockConfig();

	/*Initialize UART2*/
	uart2_init();

	/*Enable clock access to GPIOA*/
	RCC->AHB1ENR|=RCC_AHB1ENR_GPIOAEN;

	/*Set PA0 and PA1 to alternate function*/
	GPIOA->MODER|=(GPIO_MODER_MODE0_1|GPIO_MODER_MODE1_1);
	GPIOA->MODER&=~(GPIO_MODER_MODE0_0|GPIO_MODER_MODE1_0);

	GPIOA->MODER|=GPIO_MODER_MODE5_0;
	GPIOA->MODER&=~GPIO_MODER_MODE5_1;

	/*Select AF01 for TIM2*/
	#define AF01 0x01
	GPIOA->AFR[0]|=(AF01<<GPIO_AFRL_AFSEL0_Pos)|(AF01<<GPIO_AFRL_AFSEL1_Pos);

	/*enable clock access to tim2*/
	RCC->APB1ENR|=RCC_APB1ENR_TIM2EN;

	/*set the prescaler to 99 to get 1MHZ*/
	TIM2->PSC=99;

	/*set the maximum to 199999 to get 5 Hz update rate*/
	TIM2->ARR=1000000-1;

	/*Reset current counter*/
	TIM2->CNT=0;

	/*Set PA0 as PWM*/
	TIM2->CCMR1|=TIM_CCMR1_OC1M_2| TIM_CCMR1_OC1M_1;

	/*Set PA1 as Input capture*/
	TIM2->CCMR1|=TIM_CCMR1_CC2S_0;

	/*Enable CH1 and CH2*/
	TIM2->CCER|=TIM_CCER_CC1E|TIM_CCER_CC2E;


	/*Enable capture/compare 2 DMA*/
	TIM2->DIER|=TIM_DIER_CC2DE;


	/*Set CH2 to be on both edge (falling/rising edge)*/
	TIM2->CCER|=TIM_CCER_CC2P|TIM_CCER_CC2NP;

	/*DMA configuration*/

	/*Enable clock access to DMA1*/

	RCC->AHB1ENR|=RCC_AHB1ENR_DMA1EN;

	/*Disable the stream and wait until it is disabled*/
	DMA1_Stream6->CR &=~DMA_SxCR_EN;

	while((DMA1_Stream6->CR & DMA_SxCR_EN) == DMA_SxCR_EN );

	/*Configure the DMA with the following parameters
	 *
	 * Channel is channel 3
	 * Memory and peripheral size is word (32-bit)
	 * Circular mode
	 *
	 * */


	#define CH3 0x03

	DMA1_Stream6->CR |= (CH3<<DMA_SxCR_CHSEL_Pos)|DMA_SxCR_PSIZE_1|DMA_SxCR_MSIZE_1|DMA_SxCR_CIRC;

	/*Set number of transfers to be 1
	 * Peripheral address to TIM2->CCR2
	 * Memory address to be the period variable
	 * */
	DMA1_Stream6->NDTR=1;
	DMA1_Stream6->PAR= (uint32_t)&TIM2->CCR2;
	DMA1_Stream6->M0AR=(uint32_t)&period;

	/*Delay to ensure the ultrasonic sensor is up*/
	delay(30);

	/*Enable DMA*/
	DMA1_Stream6->CR |=DMA_SxCR_EN;

	/*Enable timer*/
	TIM2->CR1=TIM_CR1_CEN;

	/*Give 12 microsecond pules to trigger pin*/
	TIM2->CCR1=12;

	while(1)
	{

		/*calculate the distance*/

		distance=(float)((float)period-(float)538.0)*((float)0.034/(float)2);

		/*Print the distance*/
		printf("Distance=%0.2f cm\r\n",distance);

		/*Delay between each print*/
		delay(500);


	}

}


